// {{{RME classifier 'Logical View::VCTRL::VCOMB::Bus_Comb_Protocol'

#ifndef Bus_Comb_Protocol_H
#define Bus_Comb_Protocol_H

#ifdef PRAGMA
#pragma interface "Bus_Comb_Protocol.h"
#endif

#include <RTSystem/VANT.h>

struct Bus_Comb_Protocol
{
	class Base : public RTRootProtocol
	{
	public:
		inline Base( void );
		inline ~Base( void );
		enum
		{
			rti_consume_value = rtiLast_RTRootProtocol + 1
		  , rti_mix_value
		  , rti_level_value
		  , rti_unload_complete
		};

	protected:
		enum { rtiLast_Bus_Comb_Protocol = rti_unload_complete };

	public:
		inline RTInSignal consume_value( void );
		inline RTInSignal mix_value( void );
		inline RTInSignal level_value( void );
		inline RTInSignal unload_complete( void );
		inline RTOutSignal request_fuel_consume( void );
		inline RTOutSignal request_fuel_mix( void );
		inline RTOutSignal request_fuel_level( void );
		inline RTOutSignal do_unload( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
	class Conjugate : public RTRootProtocol
	{
	public:
		inline Conjugate( void );
		inline ~Conjugate( void );
		enum
		{
			rti_request_fuel_consume = rtiLast_RTRootProtocol + 1
		  , rti_request_fuel_mix
		  , rti_request_fuel_level
		  , rti_do_unload
		};

	protected:
		enum { rtiLast_Bus_Comb_Protocol = rti_do_unload };

	public:
		inline RTInSignal request_fuel_consume( void );
		inline RTInSignal request_fuel_mix( void );
		inline RTInSignal request_fuel_level( void );
		inline RTInSignal do_unload( void );
		inline RTOutSignal consume_value( const int & data );
		inline RTOutSignal mix_value( const bool & data );
		inline RTOutSignal level_value( const int & data );
		inline RTOutSignal unload_complete( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
};

inline Bus_Comb_Protocol::Base::Base( void )
	: RTRootProtocol()
{
}

inline Bus_Comb_Protocol::Base::~Base( void )
{
}

inline RTInSignal Bus_Comb_Protocol::Base::consume_value( void )
{
	return RTInSignal( this, rti_consume_value );
}

inline RTInSignal Bus_Comb_Protocol::Base::mix_value( void )
{
	return RTInSignal( this, rti_mix_value );
}

inline RTInSignal Bus_Comb_Protocol::Base::level_value( void )
{
	return RTInSignal( this, rti_level_value );
}

inline RTInSignal Bus_Comb_Protocol::Base::unload_complete( void )
{
	return RTInSignal( this, rti_unload_complete );
}

inline RTOutSignal Bus_Comb_Protocol::Base::request_fuel_consume( void )
{
	return RTOutSignal( this, Conjugate::rti_request_fuel_consume, (const void *)0, &RTType_void );
}

inline RTOutSignal Bus_Comb_Protocol::Base::request_fuel_mix( void )
{
	return RTOutSignal( this, Conjugate::rti_request_fuel_mix, (const void *)0, &RTType_void );
}

inline RTOutSignal Bus_Comb_Protocol::Base::request_fuel_level( void )
{
	return RTOutSignal( this, Conjugate::rti_request_fuel_level, (const void *)0, &RTType_void );
}

inline RTOutSignal Bus_Comb_Protocol::Base::do_unload( void )
{
	return RTOutSignal( this, Conjugate::rti_do_unload, (const void *)0, &RTType_void );
}

inline Bus_Comb_Protocol::Conjugate::Conjugate( void )
	: RTRootProtocol()
{
}

inline Bus_Comb_Protocol::Conjugate::~Conjugate( void )
{
}

inline RTInSignal Bus_Comb_Protocol::Conjugate::request_fuel_consume( void )
{
	return RTInSignal( this, rti_request_fuel_consume );
}

inline RTInSignal Bus_Comb_Protocol::Conjugate::request_fuel_mix( void )
{
	return RTInSignal( this, rti_request_fuel_mix );
}

inline RTInSignal Bus_Comb_Protocol::Conjugate::request_fuel_level( void )
{
	return RTInSignal( this, rti_request_fuel_level );
}

inline RTInSignal Bus_Comb_Protocol::Conjugate::do_unload( void )
{
	return RTInSignal( this, rti_do_unload );
}

inline RTOutSignal Bus_Comb_Protocol::Conjugate::consume_value( const int & data )
{
	return RTOutSignal( this, Base::rti_consume_value, &data, &RTType_int );
}

inline RTOutSignal Bus_Comb_Protocol::Conjugate::mix_value( const bool & data )
{
	return RTOutSignal( this, Base::rti_mix_value, &data, &RTType_bool );
}

inline RTOutSignal Bus_Comb_Protocol::Conjugate::level_value( const int & data )
{
	return RTOutSignal( this, Base::rti_level_value, &data, &RTType_int );
}

inline RTOutSignal Bus_Comb_Protocol::Conjugate::unload_complete( void )
{
	return RTOutSignal( this, Base::rti_unload_complete, (const void *)0, &RTType_void );
}

#endif /* Bus_Comb_Protocol_H */

// }}}RME
