// {{{RME classifier 'Logical View::VCTRL::VCONT::Bus_Cont_Protocol'

#ifndef Bus_Cont_Protocol_H
#define Bus_Cont_Protocol_H

#ifdef PRAGMA
#pragma interface "Bus_Cont_Protocol.h"
#endif

#include <RTSystem/VANT.h>

struct Bus_Cont_Protocol
{
	class Base : public RTRootProtocol
	{
	public:
		inline Base( void );
		inline ~Base( void );
		enum
		{
			rti_request_position = rtiLast_RTRootProtocol + 1
		  , rti_request_destination
		  , rti_request_controle_status
		};

	protected:
		enum { rtiLast_Bus_Cont_Protocol = rti_request_controle_status };

	public:
		inline RTInSignal request_position( void );
		inline RTInSignal request_destination( void );
		inline RTInSignal request_controle_status( void );
		inline RTOutSignal do_take_off( void );
		inline RTOutSignal position_value( const int & data );
		inline RTOutSignal destination_value( const int & data );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
	class Conjugate : public RTRootProtocol
	{
	public:
		inline Conjugate( void );
		inline ~Conjugate( void );
		enum
		{
			rti_do_take_off = rtiLast_RTRootProtocol + 1
		  , rti_position_value
		  , rti_destination_value
		};

	protected:
		enum { rtiLast_Bus_Cont_Protocol = rti_destination_value };

	public:
		inline RTInSignal do_take_off( void );
		inline RTInSignal position_value( void );
		inline RTInSignal destination_value( void );
		inline RTOutSignal request_position( void );
		inline RTOutSignal request_destination( void );
		inline RTOutSignal request_controle_status( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
};

inline Bus_Cont_Protocol::Base::Base( void )
	: RTRootProtocol()
{
}

inline Bus_Cont_Protocol::Base::~Base( void )
{
}

inline RTInSignal Bus_Cont_Protocol::Base::request_position( void )
{
	return RTInSignal( this, rti_request_position );
}

inline RTInSignal Bus_Cont_Protocol::Base::request_destination( void )
{
	return RTInSignal( this, rti_request_destination );
}

inline RTInSignal Bus_Cont_Protocol::Base::request_controle_status( void )
{
	return RTInSignal( this, rti_request_controle_status );
}

inline RTOutSignal Bus_Cont_Protocol::Base::do_take_off( void )
{
	return RTOutSignal( this, Conjugate::rti_do_take_off, (const void *)0, &RTType_void );
}

inline RTOutSignal Bus_Cont_Protocol::Base::position_value( const int & data )
{
	return RTOutSignal( this, Conjugate::rti_position_value, &data, &RTType_int );
}

inline RTOutSignal Bus_Cont_Protocol::Base::destination_value( const int & data )
{
	return RTOutSignal( this, Conjugate::rti_destination_value, &data, &RTType_int );
}

inline Bus_Cont_Protocol::Conjugate::Conjugate( void )
	: RTRootProtocol()
{
}

inline Bus_Cont_Protocol::Conjugate::~Conjugate( void )
{
}

inline RTInSignal Bus_Cont_Protocol::Conjugate::do_take_off( void )
{
	return RTInSignal( this, rti_do_take_off );
}

inline RTInSignal Bus_Cont_Protocol::Conjugate::position_value( void )
{
	return RTInSignal( this, rti_position_value );
}

inline RTInSignal Bus_Cont_Protocol::Conjugate::destination_value( void )
{
	return RTInSignal( this, rti_destination_value );
}

inline RTOutSignal Bus_Cont_Protocol::Conjugate::request_position( void )
{
	return RTOutSignal( this, Base::rti_request_position, (const void *)0, &RTType_void );
}

inline RTOutSignal Bus_Cont_Protocol::Conjugate::request_destination( void )
{
	return RTOutSignal( this, Base::rti_request_destination, (const void *)0, &RTType_void );
}

inline RTOutSignal Bus_Cont_Protocol::Conjugate::request_controle_status( void )
{
	return RTOutSignal( this, Base::rti_request_controle_status, (const void *)0, &RTType_void );
}

#endif /* Bus_Cont_Protocol_H */

// }}}RME
