// {{{RME classifier 'Logical View::VCTRL::VPOTE::Egt_Protocol'

#ifndef Egt_Protocol_H
#define Egt_Protocol_H

#ifdef PRAGMA
#pragma interface "Egt_Protocol.h"
#endif

#include <RTSystem/VANT.h>

struct Egt_Protocol
{
	class Base : public RTRootProtocol
	{
	public:
		inline Base( void );
		inline ~Base( void );
		enum { rti_egt_status = rtiLast_RTRootProtocol + 1 };

	protected:
		enum { rtiLast_Egt_Protocol = rti_egt_status };

	public:
		inline RTInSignal egt_status( void );
		inline RTOutSignal do_egt_up( void );
		inline RTOutSignal do_egt_down( void );
		inline RTOutSignal request_egt_status( void );
		inline RTOutSignal do_straight( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
	class Conjugate : public RTRootProtocol
	{
	public:
		inline Conjugate( void );
		inline ~Conjugate( void );
		enum
		{
			rti_do_egt_up = rtiLast_RTRootProtocol + 1
		  , rti_do_egt_down
		  , rti_request_egt_status
		  , rti_do_straight
		};

	protected:
		enum { rtiLast_Egt_Protocol = rti_do_straight };

	public:
		inline RTInSignal do_egt_up( void );
		inline RTInSignal do_egt_down( void );
		inline RTInSignal request_egt_status( void );
		inline RTInSignal do_straight( void );
		inline RTOutSignal egt_status( const int & data );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
};

inline Egt_Protocol::Base::Base( void )
	: RTRootProtocol()
{
}

inline Egt_Protocol::Base::~Base( void )
{
}

inline RTInSignal Egt_Protocol::Base::egt_status( void )
{
	return RTInSignal( this, rti_egt_status );
}

inline RTOutSignal Egt_Protocol::Base::do_egt_up( void )
{
	return RTOutSignal( this, Conjugate::rti_do_egt_up, (const void *)0, &RTType_void );
}

inline RTOutSignal Egt_Protocol::Base::do_egt_down( void )
{
	return RTOutSignal( this, Conjugate::rti_do_egt_down, (const void *)0, &RTType_void );
}

inline RTOutSignal Egt_Protocol::Base::request_egt_status( void )
{
	return RTOutSignal( this, Conjugate::rti_request_egt_status, (const void *)0, &RTType_void );
}

inline RTOutSignal Egt_Protocol::Base::do_straight( void )
{
	return RTOutSignal( this, Conjugate::rti_do_straight, (const void *)0, &RTType_void );
}

inline Egt_Protocol::Conjugate::Conjugate( void )
	: RTRootProtocol()
{
}

inline Egt_Protocol::Conjugate::~Conjugate( void )
{
}

inline RTInSignal Egt_Protocol::Conjugate::do_egt_up( void )
{
	return RTInSignal( this, rti_do_egt_up );
}

inline RTInSignal Egt_Protocol::Conjugate::do_egt_down( void )
{
	return RTInSignal( this, rti_do_egt_down );
}

inline RTInSignal Egt_Protocol::Conjugate::request_egt_status( void )
{
	return RTInSignal( this, rti_request_egt_status );
}

inline RTInSignal Egt_Protocol::Conjugate::do_straight( void )
{
	return RTInSignal( this, rti_do_straight );
}

inline RTOutSignal Egt_Protocol::Conjugate::egt_status( const int & data )
{
	return RTOutSignal( this, Base::rti_egt_status, &data, &RTType_int );
}

#endif /* Egt_Protocol_H */

// }}}RME
