// {{{RME classifier 'Logical View::VCTRL::VCONT::Elevator_Protocol'

#ifndef Elevator_Protocol_H
#define Elevator_Protocol_H

#ifdef PRAGMA
#pragma interface "Elevator_Protocol.h"
#endif

#include <RTSystem/VANT.h>

struct Elevator_Protocol
{
	class Base : public RTRootProtocol
	{
	public:
		inline Base( void );
		inline ~Base( void );
		enum { rti_return_elevator_status = rtiLast_RTRootProtocol + 1 };

	protected:
		enum { rtiLast_Elevator_Protocol = rti_return_elevator_status };

	public:
		inline RTInSignal return_elevator_status( void );
		inline RTOutSignal do_elevator_up( void );
		inline RTOutSignal do_elevator_down( void );
		inline RTOutSignal request_elevator_status( void );
		inline RTOutSignal do_straight( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
	class Conjugate : public RTRootProtocol
	{
	public:
		inline Conjugate( void );
		inline ~Conjugate( void );
		enum
		{
			rti_do_elevator_up = rtiLast_RTRootProtocol + 1
		  , rti_do_elevator_down
		  , rti_request_elevator_status
		  , rti_do_straight
		};

	protected:
		enum { rtiLast_Elevator_Protocol = rti_do_straight };

	public:
		inline RTInSignal do_elevator_up( void );
		inline RTInSignal do_elevator_down( void );
		inline RTInSignal request_elevator_status( void );
		inline RTInSignal do_straight( void );
		inline RTOutSignal return_elevator_status( const int & data );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
};

inline Elevator_Protocol::Base::Base( void )
	: RTRootProtocol()
{
}

inline Elevator_Protocol::Base::~Base( void )
{
}

inline RTInSignal Elevator_Protocol::Base::return_elevator_status( void )
{
	return RTInSignal( this, rti_return_elevator_status );
}

inline RTOutSignal Elevator_Protocol::Base::do_elevator_up( void )
{
	return RTOutSignal( this, Conjugate::rti_do_elevator_up, (const void *)0, &RTType_void );
}

inline RTOutSignal Elevator_Protocol::Base::do_elevator_down( void )
{
	return RTOutSignal( this, Conjugate::rti_do_elevator_down, (const void *)0, &RTType_void );
}

inline RTOutSignal Elevator_Protocol::Base::request_elevator_status( void )
{
	return RTOutSignal( this, Conjugate::rti_request_elevator_status, (const void *)0, &RTType_void );
}

inline RTOutSignal Elevator_Protocol::Base::do_straight( void )
{
	return RTOutSignal( this, Conjugate::rti_do_straight, (const void *)0, &RTType_void );
}

inline Elevator_Protocol::Conjugate::Conjugate( void )
	: RTRootProtocol()
{
}

inline Elevator_Protocol::Conjugate::~Conjugate( void )
{
}

inline RTInSignal Elevator_Protocol::Conjugate::do_elevator_up( void )
{
	return RTInSignal( this, rti_do_elevator_up );
}

inline RTInSignal Elevator_Protocol::Conjugate::do_elevator_down( void )
{
	return RTInSignal( this, rti_do_elevator_down );
}

inline RTInSignal Elevator_Protocol::Conjugate::request_elevator_status( void )
{
	return RTInSignal( this, rti_request_elevator_status );
}

inline RTInSignal Elevator_Protocol::Conjugate::do_straight( void )
{
	return RTInSignal( this, rti_do_straight );
}

inline RTOutSignal Elevator_Protocol::Conjugate::return_elevator_status( const int & data )
{
	return RTOutSignal( this, Base::rti_return_elevator_status, &data, &RTType_int );
}

#endif /* Elevator_Protocol_H */

// }}}RME
