// {{{RME classifier 'Logical View::VCTRL::VCONT::Flap'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Flap.h"
#endif

#include <RTSystem/VANT.h>
#include <Flap.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_controller"
	  , &Flap_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Flap_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Flap_Actor( _rts, _ref );
}

const RTActorClass Flap =
{
	(const RTActorClass *)0
  , "Flap"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_Flap_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Active"
};

#define SUPER RTActor

Flap_Actor::Flap_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, flap_status( 1 )
{
}

Flap_Actor::~Flap_Actor( void )
{
}

int Flap_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_controller
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_controller;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Active:J4359EB9D0179:request_flap'
INLINE_METHODS void Flap_Actor::transition2_request_flap( const void * rtdata, Flap_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_controller.flap_grade_status(flap_status).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359EC220263:go_down'
INLINE_METHODS void Flap_Actor::transition3_go_down( const void * rtdata, Flap_Protocol::Conjugate * rtport )
{
	// {{{USR
	flap_status=0;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359EC3E0196:go_straight'
INLINE_METHODS void Flap_Actor::transition4_go_straight( const void * rtdata, Flap_Protocol::Conjugate * rtport )
{
	// {{{USR
	flap_status=1;
	// }}}USR
}
// }}}RME

INLINE_CHAINS void Flap_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Flap_Actor::chain3_go_down( void )
{
	// transition ':TOP:Active:J4359EC220263:go_down'
	rtgChainBegin( 2, "go_down" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_go_down( msg->data, (Flap_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Flap_Actor::chain2_request_flap( void )
{
	// transition ':TOP:Active:J4359EB9D0179:request_flap'
	rtgChainBegin( 2, "request_flap" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_request_flap( msg->data, (Flap_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Flap_Actor::chain4_go_straight( void )
{
	// transition ':TOP:Active:J4359EC3E0196:go_straight'
	rtgChainBegin( 2, "go_straight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_go_straight( msg->data, (Flap_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void Flap_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Active'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_controller'
				switch( signalIndex )
				{
				case Flap_Protocol::Conjugate::rti_do_flap_down:
					chain3_go_down();
					return;
				case Flap_Protocol::Conjugate::rti_request_flap_position:
					chain2_request_flap();
					return;
				case Flap_Protocol::Conjugate::rti_do_flap_straight:
					chain4_go_straight();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Flap_Actor::getActorData( void ) const
{
	return &Flap_Actor::rtg_class;
}

const RTActor_class Flap_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 2
  , Flap_Actor::rtg_parent_state
  , &Flap
  , 0
  , (const RTComponentDescriptor *)0
  , 1
  , Flap_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , Flap_Actor::rtg_Flap_fields
};

const RTStateId Flap_Actor::rtg_parent_state[] =
{
	0
  , 1
};

const RTPortDescriptor Flap_Actor::rtg_ports[] =
{
	{
		"to_controller"
	  , (const char *)0
	  , &Flap_Protocol::Conjugate::rt_class
	  , RTOffsetOf( Flap_Actor, Flap_Actor::to_controller )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor Flap_Actor::rtg_Flap_fields[] =
{
	// {{{RME classAttribute 'flap_status'
	{
		"flap_status"
	  , RTOffsetOf( Flap_Actor, flap_status )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_bool
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
