// {{{RME classifier 'Logical View::VCTRL::VCONT::FlightControl'

#ifndef FlightControl_H
#define FlightControl_H

#ifdef PRAGMA
#pragma interface "FlightControl.h"
#endif

#include <RTSystem/VANT.h>
#include <Aileron_Protocol.h>
#include <Bus_Cont_Protocol.h>
#include <Elevator_Protocol.h>
#include <Flap_Protocol.h>
#include <Power_Cont_Protocol.h>
#include <Rudder_Protocol.h>

// {{{RME tool 'OT::Cpp' property 'HeaderPreface'
// {{{USR
#include <stdio.h>
// }}}USR
// }}}RME

extern const RTActorClass FlightControl;

#define SUPER RTActor
class FlightControl_Actor : public RTActor
{
public:
	// {{{RME tool 'OT::Cpp' property 'PublicDeclarations'
	// {{{USR

	// }}}USR
	// }}}RME

protected:
	// {{{RME tool 'OT::Cpp' property 'ProtectedDeclarations'
	// {{{USR

	// }}}USR
	// }}}RME

private:
	// {{{RME tool 'OT::Cpp' property 'PrivateDeclarations'
	// {{{USR

	// }}}USR
	// }}}RME
	// {{{RME classAttribute 'position'
	int position;
	// }}}RME
	// {{{RME classAttribute 'destination'
	int destination;
	// }}}RME

protected:
	// {{{RME port 'to_bus'
	Bus_Cont_Protocol::Conjugate to_bus;
	// }}}RME
	// {{{RME port 'to_power'
	Power_Cont_Protocol::Conjugate to_power;
	// }}}RME
	// {{{RME port 'to_aileron'
	Aileron_Protocol::Base to_aileron;
	// }}}RME
	// {{{RME port 'to_elevator'
	Elevator_Protocol::Base to_elevator;
	// }}}RME
	// {{{RME port 'to_flap'
	Flap_Protocol::Base to_flap;
	// }}}RME
	// {{{RME port 'to_rudder'
	Rudder_Protocol::Base to_rudder;
	// }}}RME
	// {{{RME port 'timer1'
	Timing::Base timer1;
	// }}}RME
	// {{{RME port 'timer2'
	Timing::Base timer2;
	// }}}RME

public:
	FlightControl_Actor( RTController * rtg_rts, RTActorRef * rtg_ref );
	virtual ~FlightControl_Actor( void );
	virtual int _followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex );

protected:
	// {{{RME enter ':TOP:IDLE'
	INLINE_METHODS void enter2_IDLE( void );
	// }}}RME
	virtual void enterStateV( void );
	// {{{RME transition ':TOP:IDLE:J4359EDC60139:to_start'
	INLINE_METHODS void transition2_to_start( const void * rtdata, Bus_Cont_Protocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:START:J4359F592007E:to_taking_off'
	INLINE_METHODS void transition3_to_taking_off( const void * rtdata, Timing::Base * rtport );
	// }}}RME
	// {{{RME transition ':TOP:TAKING_OFF:J4359F64A02C0:to_normal'
	INLINE_METHODS void transition4_to_normal( const void * rtdata, Timing::Base * rtport );
	// }}}RME
	// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359F7890224:timer_pos'
	INLINE_METHODS void transition5_timer_pos( const void * rtdata, Timing::Base * rtport );
	// }}}RME
	// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359F81E00EC:position'
	INLINE_METHODS void transition6_position( const int * rtdata, Bus_Cont_Protocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359F8B80040:destination'
	INLINE_METHODS void transition7_destination( const int * rtdata, Bus_Cont_Protocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359FA2F02CE:to_landing'
	INLINE_METHODS void transition8_to_landing( const void * rtdata, Timing::Base * rtport );
	// }}}RME
	// {{{RME transition ':TOP:IDLE:J437F843502E0:self'
	INLINE_METHODS void transition10_self( const int * rtdata, Bus_Cont_Protocol::Conjugate * rtport );
	// }}}RME

private:
	INLINE_CHAINS void chain1_Initial( void );
	INLINE_CHAINS void chain2_to_start( void );
	INLINE_CHAINS void chain10_self( void );
	INLINE_CHAINS void chain3_to_taking_off( void );
	INLINE_CHAINS void chain4_to_normal( void );
	INLINE_CHAINS void chain6_position( void );
	INLINE_CHAINS void chain7_destination( void );
	INLINE_CHAINS void chain8_to_landing( void );
	INLINE_CHAINS void chain5_timer_pos( void );
	INLINE_CHAINS void chain9_t1( void );

public:
	virtual void rtsBehavior( int signalIndex, int portIndex );
	virtual const RTActor_class * getActorData( void ) const;

protected:
	static const RTActor_class rtg_class;

public:
	static const RTStateId rtg_parent_state[];

private:
	static const RTPortDescriptor rtg_ports[];

public:
	static const RTFieldDescriptor rtg_FlightControl_fields[];
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'HeaderEnding'
// {{{USR

// }}}USR
// }}}RME

#endif /* FlightControl_H */

// }}}RME
