// {{{RME classifier 'Logical View::VCNS::VVIG::Image_Controller'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Image_Controller.h"
#endif

#include <RTSystem/VANT.h>
#include <Image_Controller.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_intbus"
	  , &VIG_PROTOCOL::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"to_Image_Device"
	  , &Image_Device_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_Image_Processor"
	  , &Image_Processor_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_Image_Classifier"
	  , &Image_Classifier_Protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Image_Controller_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Image_Controller_Actor( _rts, _ref );
}

const RTActorClass Image_Controller =
{
	(const RTActorClass *)0
  , "Image_Controller"
  , (RTVersionId)0
  , 4
  , rtg_relays
  , new_Image_Controller_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Wating"
  , "Image_Capture"
  , "Image_Process"
  , "Image_Classify"
};

#define SUPER RTActor

Image_Controller_Actor::Image_Controller_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

Image_Controller_Actor::~Image_Controller_Actor( void )
{
}

int Image_Controller_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_intbus
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_intbus;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// to_Image_Device
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_Image_Device;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 2:
		// to_Image_Processor
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_Image_Processor;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 3:
		// to_Image_Classifier
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_Image_Classifier;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME enter ':TOP:Image_Capture'
INLINE_METHODS void Image_Controller_Actor::enter3_Image_Capture( void )
{
	// {{{USR
	to_Image_Device.take_photo().send();
	// }}}USR
}
// }}}RME

void Image_Controller_Actor::enterStateV( void )
{
	switch( getCurrentState() )
	{
	case 3:
		enter3_Image_Capture();
		break;
	case 4:
		enter4_Image_Process();
		break;
	case 5:
		enter5_Image_Classify();
		break;
	default:
		RTActor::enterStateV();
		break;
	}
}

// {{{RME enter ':TOP:Image_Process'
INLINE_METHODS void Image_Controller_Actor::enter4_Image_Process( void )
{
	// {{{USR
	to_Image_Processor.Process_Image().send();
	// }}}USR
}
// }}}RME

// {{{RME enter ':TOP:Image_Classify'
INLINE_METHODS void Image_Controller_Actor::enter5_Image_Classify( void )
{
	// {{{USR
	to_Image_Classifier.Classify_Image().send();
	// }}}USR
}
// }}}RME

INLINE_CHAINS void Image_Controller_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Image_Controller_Actor::chain2_to_Capture( void )
{
	// transition ':TOP:Wating:J43679D8C0179:to_Capture'
	rtgChainBegin( 2, "to_Capture" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void Image_Controller_Actor::chain3_to_Process( void )
{
	// transition ':TOP:Image_Capture:J43679DC90157:to_Process'
	rtgChainBegin( 3, "to_Process" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void Image_Controller_Actor::chain4_To_Classification( void )
{
	// transition ':TOP:Image_Process:J43679DCC0188:To_Classification'
	rtgChainBegin( 4, "To_Classification" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void Image_Controller_Actor::chain5_Wait( void )
{
	// transition ':TOP:Image_Classify:J43679DCF00CD:Wait'
	rtgChainBegin( 5, "Wait" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

void Image_Controller_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Wating'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_intbus'
				switch( signalIndex )
				{
				case VIG_PROTOCOL::Conjugate::rti_Captar_Imagem:
					chain2_to_Capture();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:Image_Capture'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'to_Image_Device'
				switch( signalIndex )
				{
				case Image_Device_Protocol::Base::rti_photo_ready:
					chain3_to_Process();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:Image_Process'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 3:
				// {{{RME port 'to_Image_Processor'
				switch( signalIndex )
				{
				case Image_Processor_Protocol::Base::rti_Image_Processed:
					chain4_To_Classification();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME state ':TOP:Image_Classify'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 4:
				// {{{RME port 'to_Image_Classifier'
				switch( signalIndex )
				{
				case Image_Classifier_Protocol::Base::rti_Image_Classified:
					chain5_Wait();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Image_Controller_Actor::getActorData( void ) const
{
	return &Image_Controller_Actor::rtg_class;
}

const RTActor_class Image_Controller_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 5
  , Image_Controller_Actor::rtg_parent_state
  , &Image_Controller
  , 0
  , (const RTComponentDescriptor *)0
  , 4
  , Image_Controller_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId Image_Controller_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
  , 1
};

const RTPortDescriptor Image_Controller_Actor::rtg_ports[] =
{
	{
		"to_intbus"
	  , (const char *)0
	  , &VIG_PROTOCOL::Conjugate::rt_class
	  , RTOffsetOf( Image_Controller_Actor, Image_Controller_Actor::to_intbus )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_Image_Device"
	  , (const char *)0
	  , &Image_Device_Protocol::Base::rt_class
	  , RTOffsetOf( Image_Controller_Actor, Image_Controller_Actor::to_Image_Device )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_Image_Processor"
	  , (const char *)0
	  , &Image_Processor_Protocol::Base::rt_class
	  , RTOffsetOf( Image_Controller_Actor, Image_Controller_Actor::to_Image_Processor )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_Image_Classifier"
	  , (const char *)0
	  , &Image_Classifier_Protocol::Base::rt_class
	  , RTOffsetOf( Image_Controller_Actor, Image_Controller_Actor::to_Image_Classifier )
	  , 1 // cardinality
	  , 4
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
