// {{{RME classifier 'Logical View::VCTRL::VCOMB::Level'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Level.h"
#endif

#include <RTSystem/VANT.h>
#include <Level.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_controller"
	  , &Level_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Level_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Level_Actor( _rts, _ref );
}

const RTActorClass Level =
{
	(const RTActorClass *)0
  , "Level"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_Level_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Active"
};

#define SUPER RTActor

Level_Actor::Level_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, level_value( 1000 )
{
}

Level_Actor::~Level_Actor( void )
{
}

int Level_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_controller
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_controller;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Active:J43599F9403A9:request_level'
INLINE_METHODS void Level_Actor::transition1_request_level( const void * rtdata, Level_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_controller.level_value(level_value).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359A17E0177:engine_on'
INLINE_METHODS void Level_Actor::transition3_engine_on( const void * rtdata, Level_Protocol::Conjugate * rtport )
{
	// {{{USR
	tid = timer.informEvery( RTTimespec( 20, 0 ) );
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359A2080253:decrement'
INLINE_METHODS void Level_Actor::transition4_decrement( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	level_value = level_value - level_factor;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359BF33039B:unload'
INLINE_METHODS void Level_Actor::transition5_unload( const void * rtdata, Level_Protocol::Conjugate * rtport )
{
	// {{{USR
	level_value=0;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359C73B03AB:engine_off'
INLINE_METHODS void Level_Actor::transition6_engine_off( const void * rtdata, Level_Protocol::Conjugate * rtport )
{
	// {{{USR
	timer.cancelTimer( tid );
	// }}}USR
}
// }}}RME

INLINE_CHAINS void Level_Actor::chain2_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Level_Actor::chain1_request_level( void )
{
	// transition ':TOP:Active:J43599F9403A9:request_level'
	rtgChainBegin( 2, "request_level" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition1_request_level( msg->data, (Level_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Level_Actor::chain3_engine_on( void )
{
	// transition ':TOP:Active:J4359A17E0177:engine_on'
	rtgChainBegin( 2, "engine_on" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_engine_on( msg->data, (Level_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Level_Actor::chain5_unload( void )
{
	// transition ':TOP:Active:J4359BF33039B:unload'
	rtgChainBegin( 2, "unload" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_unload( msg->data, (Level_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Level_Actor::chain6_engine_off( void )
{
	// transition ':TOP:Active:J4359C73B03AB:engine_off'
	rtgChainBegin( 2, "engine_off" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_engine_off( msg->data, (Level_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Level_Actor::chain4_decrement( void )
{
	// transition ':TOP:Active:J4359A2080253:decrement'
	rtgChainBegin( 2, "decrement" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_decrement( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void Level_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain2_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Active'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_controller'
				switch( signalIndex )
				{
				case Level_Protocol::Conjugate::rti_request_fuel_level:
					chain1_request_level();
					return;
				case Level_Protocol::Conjugate::rti_engine_on:
					chain3_engine_on();
					return;
				case Level_Protocol::Conjugate::rti_unload:
					chain5_unload();
					return;
				case Level_Protocol::Conjugate::rti_engine_off:
					chain6_engine_off();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain4_decrement();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Level_Actor::getActorData( void ) const
{
	return &Level_Actor::rtg_class;
}

const RTActor_class Level_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 2
  , Level_Actor::rtg_parent_state
  , &Level
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , Level_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 2
  , Level_Actor::rtg_Level_fields
};

const RTStateId Level_Actor::rtg_parent_state[] =
{
	0
  , 1
};

const RTPortDescriptor Level_Actor::rtg_ports[] =
{
	{
		"to_controller"
	  , (const char *)0
	  , &Level_Protocol::Conjugate::rt_class
	  , RTOffsetOf( Level_Actor, Level_Actor::to_controller )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( Level_Actor, Level_Actor::timer )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

const RTFieldDescriptor Level_Actor::rtg_Level_fields[] =
{
	// {{{RME classAttribute 'level_value'
	{
		"level_value"
	  , RTOffsetOf( Level_Actor, level_value )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'level_factor'
  , {
		"level_factor"
	  , RTOffsetOf( Level_Actor, level_factor )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
