// {{{RME classifier 'Logical View::VCTRL::VCOMB::Mix_Protocol'

#ifndef Mix_Protocol_H
#define Mix_Protocol_H

#ifdef PRAGMA
#pragma interface "Mix_Protocol.h"
#endif

#include <RTSystem/VANT.h>

struct Mix_Protocol
{
	class Base : public RTRootProtocol
	{
	public:
		inline Base( void );
		inline ~Base( void );
		enum { rti_fuel_mix_value = rtiLast_RTRootProtocol + 1 };

	protected:
		enum { rtiLast_Mix_Protocol = rti_fuel_mix_value };

	public:
		inline RTInSignal fuel_mix_value( void );
		inline RTOutSignal request_fuel_mix( void );
		inline RTOutSignal active_mix( void );
		inline RTOutSignal deactive_mix( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
	class Conjugate : public RTRootProtocol
	{
	public:
		inline Conjugate( void );
		inline ~Conjugate( void );
		enum
		{
			rti_request_fuel_mix = rtiLast_RTRootProtocol + 1
		  , rti_active_mix
		  , rti_deactive_mix
		};

	protected:
		enum { rtiLast_Mix_Protocol = rti_deactive_mix };

	public:
		inline RTInSignal request_fuel_mix( void );
		inline RTInSignal active_mix( void );
		inline RTInSignal deactive_mix( void );
		inline RTOutSignal fuel_mix_value( const bool & data );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
};

inline Mix_Protocol::Base::Base( void )
	: RTRootProtocol()
{
}

inline Mix_Protocol::Base::~Base( void )
{
}

inline RTInSignal Mix_Protocol::Base::fuel_mix_value( void )
{
	return RTInSignal( this, rti_fuel_mix_value );
}

inline RTOutSignal Mix_Protocol::Base::request_fuel_mix( void )
{
	return RTOutSignal( this, Conjugate::rti_request_fuel_mix, (const void *)0, &RTType_void );
}

inline RTOutSignal Mix_Protocol::Base::active_mix( void )
{
	return RTOutSignal( this, Conjugate::rti_active_mix, (const void *)0, &RTType_void );
}

inline RTOutSignal Mix_Protocol::Base::deactive_mix( void )
{
	return RTOutSignal( this, Conjugate::rti_deactive_mix, (const void *)0, &RTType_void );
}

inline Mix_Protocol::Conjugate::Conjugate( void )
	: RTRootProtocol()
{
}

inline Mix_Protocol::Conjugate::~Conjugate( void )
{
}

inline RTInSignal Mix_Protocol::Conjugate::request_fuel_mix( void )
{
	return RTInSignal( this, rti_request_fuel_mix );
}

inline RTInSignal Mix_Protocol::Conjugate::active_mix( void )
{
	return RTInSignal( this, rti_active_mix );
}

inline RTInSignal Mix_Protocol::Conjugate::deactive_mix( void )
{
	return RTInSignal( this, rti_deactive_mix );
}

inline RTOutSignal Mix_Protocol::Conjugate::fuel_mix_value( const bool & data )
{
	return RTOutSignal( this, Base::rti_fuel_mix_value, &data, &RTType_bool );
}

#endif /* Mix_Protocol_H */

// }}}RME
