// {{{RME classifier 'Logical View::VCTRL::VCOMB::Power_Comb_Protocol'

#ifndef Power_Comb_Protocol_H
#define Power_Comb_Protocol_H

#ifdef PRAGMA
#pragma interface "Power_Comb_Protocol.h"
#endif

#include <RTSystem/VANT.h>

struct Power_Comb_Protocol
{
	class Base : public RTRootProtocol
	{
	public:
		inline Base( void );
		inline ~Base( void );
		enum
		{
			rti_consume_value = rtiLast_RTRootProtocol + 1
		  , rti_level_value
		  , rti_mix_value
		  , rti_flow_increased
		  , rti_flow_decreased
		};

	protected:
		enum { rtiLast_Power_Comb_Protocol = rti_flow_decreased };

	public:
		inline RTInSignal consume_value( void );
		inline RTInSignal level_value( void );
		inline RTInSignal mix_value( void );
		inline RTInSignal flow_increased( void );
		inline RTInSignal flow_decreased( void );
		inline RTOutSignal request_fuel_level( void );
		inline RTOutSignal request_fuel_consume( void );
		inline RTOutSignal request_fuel_mix( void );
		inline RTOutSignal increase_flow( void );
		inline RTOutSignal decrease_flow( void );
		inline RTOutSignal engine_on( void );
		inline RTOutSignal engine_off( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
	class Conjugate : public RTRootProtocol
	{
	public:
		inline Conjugate( void );
		inline ~Conjugate( void );
		enum
		{
			rti_request_fuel_level = rtiLast_RTRootProtocol + 1
		  , rti_request_fuel_consume
		  , rti_request_fuel_mix
		  , rti_increase_flow
		  , rti_decrease_flow
		  , rti_engine_on
		  , rti_engine_off
		};

	protected:
		enum { rtiLast_Power_Comb_Protocol = rti_engine_off };

	public:
		inline RTInSignal request_fuel_level( void );
		inline RTInSignal request_fuel_consume( void );
		inline RTInSignal request_fuel_mix( void );
		inline RTInSignal increase_flow( void );
		inline RTInSignal decrease_flow( void );
		inline RTInSignal engine_on( void );
		inline RTInSignal engine_off( void );
		inline RTOutSignal consume_value( const int & data );
		inline RTOutSignal level_value( const int & data );
		inline RTOutSignal mix_value( const bool & data );
		inline RTOutSignal flow_increased( void );
		inline RTOutSignal flow_decreased( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
};

inline Power_Comb_Protocol::Base::Base( void )
	: RTRootProtocol()
{
}

inline Power_Comb_Protocol::Base::~Base( void )
{
}

inline RTInSignal Power_Comb_Protocol::Base::consume_value( void )
{
	return RTInSignal( this, rti_consume_value );
}

inline RTInSignal Power_Comb_Protocol::Base::level_value( void )
{
	return RTInSignal( this, rti_level_value );
}

inline RTInSignal Power_Comb_Protocol::Base::mix_value( void )
{
	return RTInSignal( this, rti_mix_value );
}

inline RTInSignal Power_Comb_Protocol::Base::flow_increased( void )
{
	return RTInSignal( this, rti_flow_increased );
}

inline RTInSignal Power_Comb_Protocol::Base::flow_decreased( void )
{
	return RTInSignal( this, rti_flow_decreased );
}

inline RTOutSignal Power_Comb_Protocol::Base::request_fuel_level( void )
{
	return RTOutSignal( this, Conjugate::rti_request_fuel_level, (const void *)0, &RTType_void );
}

inline RTOutSignal Power_Comb_Protocol::Base::request_fuel_consume( void )
{
	return RTOutSignal( this, Conjugate::rti_request_fuel_consume, (const void *)0, &RTType_void );
}

inline RTOutSignal Power_Comb_Protocol::Base::request_fuel_mix( void )
{
	return RTOutSignal( this, Conjugate::rti_request_fuel_mix, (const void *)0, &RTType_void );
}

inline RTOutSignal Power_Comb_Protocol::Base::increase_flow( void )
{
	return RTOutSignal( this, Conjugate::rti_increase_flow, (const void *)0, &RTType_void );
}

inline RTOutSignal Power_Comb_Protocol::Base::decrease_flow( void )
{
	return RTOutSignal( this, Conjugate::rti_decrease_flow, (const void *)0, &RTType_void );
}

inline RTOutSignal Power_Comb_Protocol::Base::engine_on( void )
{
	return RTOutSignal( this, Conjugate::rti_engine_on, (const void *)0, &RTType_void );
}

inline RTOutSignal Power_Comb_Protocol::Base::engine_off( void )
{
	return RTOutSignal( this, Conjugate::rti_engine_off, (const void *)0, &RTType_void );
}

inline Power_Comb_Protocol::Conjugate::Conjugate( void )
	: RTRootProtocol()
{
}

inline Power_Comb_Protocol::Conjugate::~Conjugate( void )
{
}

inline RTInSignal Power_Comb_Protocol::Conjugate::request_fuel_level( void )
{
	return RTInSignal( this, rti_request_fuel_level );
}

inline RTInSignal Power_Comb_Protocol::Conjugate::request_fuel_consume( void )
{
	return RTInSignal( this, rti_request_fuel_consume );
}

inline RTInSignal Power_Comb_Protocol::Conjugate::request_fuel_mix( void )
{
	return RTInSignal( this, rti_request_fuel_mix );
}

inline RTInSignal Power_Comb_Protocol::Conjugate::increase_flow( void )
{
	return RTInSignal( this, rti_increase_flow );
}

inline RTInSignal Power_Comb_Protocol::Conjugate::decrease_flow( void )
{
	return RTInSignal( this, rti_decrease_flow );
}

inline RTInSignal Power_Comb_Protocol::Conjugate::engine_on( void )
{
	return RTInSignal( this, rti_engine_on );
}

inline RTInSignal Power_Comb_Protocol::Conjugate::engine_off( void )
{
	return RTInSignal( this, rti_engine_off );
}

inline RTOutSignal Power_Comb_Protocol::Conjugate::consume_value( const int & data )
{
	return RTOutSignal( this, Base::rti_consume_value, &data, &RTType_int );
}

inline RTOutSignal Power_Comb_Protocol::Conjugate::level_value( const int & data )
{
	return RTOutSignal( this, Base::rti_level_value, &data, &RTType_int );
}

inline RTOutSignal Power_Comb_Protocol::Conjugate::mix_value( const bool & data )
{
	return RTOutSignal( this, Base::rti_mix_value, &data, &RTType_bool );
}

inline RTOutSignal Power_Comb_Protocol::Conjugate::flow_increased( void )
{
	return RTOutSignal( this, Base::rti_flow_increased, (const void *)0, &RTType_void );
}

inline RTOutSignal Power_Comb_Protocol::Conjugate::flow_decreased( void )
{
	return RTOutSignal( this, Base::rti_flow_decreased, (const void *)0, &RTType_void );
}

#endif /* Power_Comb_Protocol_H */

// }}}RME
