// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTAbstractTestBase'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "RQARTAbstractTestBase.h"
#endif

#include <RTSystem/VANT.h>
#include <RQARTAbstractTestBase.h>
#include <RQARTIncarnateArguments.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static RTActor * new_RQARTAbstractTestBase_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new RQARTAbstractTestBase_Actor( _rts, _ref );
}

const RTActorClass RQARTAbstractTestBase =
{
	(const RTActorClass *)0
  , "RQARTAbstractTestBase"
  , (RTVersionId)0
  , 0
  , (const RTRelayDescriptor *)0
  , new_RQARTAbstractTestBase_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
};

#define SUPER RTActor

RQARTAbstractTestBase_Actor::RQARTAbstractTestBase_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

RQARTAbstractTestBase_Actor::~RQARTAbstractTestBase_Actor( void )
{
}

// {{{RME operation 'CreateCapsuleRole(RTActorRef &,const RQARTIncarnateArguments *)'
bool RQARTAbstractTestBase_Actor::CreateCapsuleRole( RTActorRef & capsuleRole, const RQARTIncarnateArguments * args )
{
	// {{{USR
	// incarnate the specified role based on the incarnation arguments

	const RTActorClass *pActorClass;
	RTActorId capsuleId;

	if(!args)
		capsuleId = RQART_Frame.incarnate(capsuleRole);
	else
	{
		if(args->pCapsuleClass == (RTActorClass*)0)
		{
			// default to class of the role
			pActorClass = &EmptyActorClass;
		}
		else
			pActorClass = args->pCapsuleClass;

	    capsuleId = RQART_Frame.incarnate(capsuleRole, *pActorClass, args->pData,
			args->pType, args->pThread, args->cardinalityIndex);
	}

	if(!capsuleId.isValid())
	{
		RQARTExceptionCode exCode = eRQARTIncarnateError;
		RQART_Exception.error(RTTypedValue(&exCode)).raise();
		return false;
	}
	else
		return true;

	// }}}USR
}
// }}}RME

// {{{RME operation 'DestroyCapsuleRole(RTActorRef &,int)'
bool RQARTAbstractTestBase_Actor::DestroyCapsuleRole( RTActorRef & capsuleRole, int cardinalityIndex )
{
	// {{{USR
	if (cardinalityIndex == -1)
	{
		// Destroy all instances of the capsuleRole
		if (!RQART_Frame.destroy(capsuleRole))
			return false;
		else
			return true;
	}
	else
	{
		// Destroy a specific instance of the capsuleRole
		RTActorId capsuleId = RQART_Frame.incarnationAt(capsuleRole, cardinalityIndex);
		if(!capsuleId.isValid())
			return false; // destroy considered failed if not valid
		if (!RQART_Frame.destroy(capsuleId))
			return false; // destroy failed
		else
			return true;  // destroy succeeded
	}

	// }}}USR
}
// }}}RME

void RQARTAbstractTestBase_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * RQARTAbstractTestBase_Actor::getActorData( void ) const
{
	return &RQARTAbstractTestBase_Actor::rtg_class;
}

const RTActor_class RQARTAbstractTestBase_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 1
  , RQARTAbstractTestBase_Actor::rtg_parent_state
  , &RQARTAbstractTestBase
  , 0
  , (const RTComponentDescriptor *)0
  , 3
  , RQARTAbstractTestBase_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId RQARTAbstractTestBase_Actor::rtg_parent_state[] =
{
	0
};

const RTPortDescriptor RQARTAbstractTestBase_Actor::rtg_ports[] =
{
	{
		"RQART_Exception"
	  , (const char *)0
	  , &Exception::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestBase_Actor, RQARTAbstractTestBase_Actor::RQART_Exception )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Frame"
	  , (const char *)0
	  , &Frame::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestBase_Actor, RQARTAbstractTestBase_Actor::RQART_Frame )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Log"
	  , (const char *)0
	  , &Log::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestBase_Actor, RQARTAbstractTestBase_Actor::RQART_Log )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
