// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTTCPServer'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "RQARTTCPServer.h"
#endif

#include <RTSystem/VANT.h>
#include <RQARTTCPServer.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"AddIn"
	  , &RQARTAddInProtocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_RQARTTCPServer_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new RQARTTCPServer_Actor( _rts, _ref );
}

const RTActorClass RQARTTCPServer =
{
	(const RTActorClass *)0
  , "RQARTTCPServer"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_RQARTTCPServer_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Operational"
  , "Failed"
  , "WaitForPort"
  , "WaitForConnection"
  , "HaveListener"
  , "HaveConnection"
};

const RTTypeModifier rtg_tm_RQARTTCPServer_Actor_anBuffer =
{
	RTNumberConstant
  , (TS_MAXNUMS)
  , 0
};

const RTTypeModifier rtg_tm_RQARTTCPServer_Actor_readBuffer =
{
	RTNumberConstant
  , (TS_MAXSIZE)
  , 0
};

#define SUPER RTActor

RQARTTCPServer_Actor::RQARTTCPServer_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, nPort( 27111 )
	, nSizeRemaining( sizeof(unsigned long) )
	, nBodyRemaining( -1 )
{
}

RQARTTCPServer_Actor::~RQARTTCPServer_Actor( void )
{
}

// {{{RME operation 'SendInt(char,int)'
void RQARTTCPServer_Actor::SendInt( char cMessage, int nData )
{
	// {{{USR
	const int nWriteSize = 2 * sizeof(unsigned long) + 1;
	const unsigned long lOne = 1;
	char *dataBuffer = new char[nWriteSize];

	unsigned long lData;
	lData = (unsigned long)nData;
	RTMemoryUtil::memcpy(dataBuffer, &lOne, sizeof(unsigned long));
	dataBuffer[sizeof(unsigned long)] = cMessage;
	RTMemoryUtil::memcpy(dataBuffer + sizeof(unsigned long) + 1, &lData, sizeof(unsigned long));

	c_socket.write(dataBuffer, nWriteSize);
	delete [] dataBuffer;

	// }}}USR
}
// }}}RME

// {{{RME operation 'ReceiveMessage()'
int RQARTTCPServer_Actor::ReceiveMessage( void )
{
	// {{{USR
	int nRead;
	if (nSizeRemaining)
	{
		// Each message starts with an unsigned long that is the number of bytes
		// in the body of the message.
		nRead = c_socket.read(readBuffer + sizeof(unsigned long) - nSizeRemaining,
			nSizeRemaining);
		if(nRead <= 0)
			return IPCReturn(0, false);	// read not finished, no reset

		nSizeRemaining -= nRead;
		if (nSizeRemaining == 0)
		{
			// Read of unsigned long complete
			RTMemoryUtil::memcpy(&nNums, readBuffer, sizeof(unsigned long));
			if(nNums > TS_MAXNUMS)
			{
				// This is either an internal error or a TCP/IP failure.
				// Shutdown the test, log an error and stop polling for messages.
				AddIn.Reset().send();
				RQART_Log.log("RQARTTCPServer detected an error. Please cancel the test.");
				return IPCReturn(0, false, false); // failed, no reset, no timeout
			}
			nBodyRemaining = TS_MESSAGE_SIZE(nNums);
			nSize = nBodyRemaining;
		}
	}

	if (nBodyRemaining != -1)
	{
		// Read the message body
		nRead = c_socket.read(readBuffer + nSize - nBodyRemaining, nBodyRemaining);
		if(nRead <= 0)
			return IPCReturn(0, false);

		nBodyRemaining -= nRead;
		if (nBodyRemaining == 0)
		{
			// Read of message complete
			anBuffer[0] = readBuffer[0];
			for(int i=1; i <= nNums ; i++)
			{
				unsigned long start = (i-1) * sizeof(unsigned long) + 1;
				unsigned long wData;
				RTMemoryUtil::memcpy(&wData, readBuffer + start, sizeof(unsigned long));
				anBuffer[i] = wData;
			}
			return IPCReturn(1, true);	// read finished, reset
		}
	}
	return IPCReturn(0, false);	// read not finished, no reset
	// }}}USR
}
// }}}RME

// {{{RME operation 'SendChars(char,char *,int)'
void RQARTTCPServer_Actor::SendChars( char cMessage, char * pcData, int nSize )
{
	// {{{USR
	const int nWriteSize = sizeof(unsigned long) + 1 + nSize;
	char *dataBuffer = new char[nWriteSize];

	long lSize = (unsigned long)nSize;
	RTMemoryUtil::memcpy(dataBuffer, &lSize, sizeof(unsigned long));
	dataBuffer[sizeof(unsigned long)] = cMessage;
	RTMemoryUtil::memcpy(dataBuffer + sizeof(unsigned long) + 1, pcData, nSize);

	c_socket.write(dataBuffer, nWriteSize);
	delete [] dataBuffer;

	// }}}USR
}
// }}}RME

// {{{RME operation 'IPCReturn(int,bool,bool)'
int RQARTTCPServer_Actor::IPCReturn( int nReturn, bool bReset, bool bSetTimer )
{
	// {{{USR
	// Helper function called by ReceiveMessage
	if (bReset)
	{
		nSizeRemaining = sizeof(unsigned long);
		nBodyRemaining = -1;
	}
	if (bSetTimer)
		RQART_Timer.informIn(RTTimespec(0, 200000000));

	return nReturn;
	// }}}USR
}
// }}}RME

int RQARTTCPServer_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// AddIn
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &AddIn;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:WaitForPort:Junction2:SetPort'
INLINE_METHODS void RQARTTCPServer_Actor::transition3_SetPort( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	nPort = *rtdata;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:HaveListener:True'
INLINE_METHODS void RQARTTCPServer_Actor::transition4_True( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	RQART_Timer.informIn(RTTimespec(0, 1));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:HaveConnection:True'
INLINE_METHODS void RQARTTCPServer_Actor::transition6_True( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	RQART_Timer.informIn(RTTimespec(0, 200000000));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:HaveConnection:False'
INLINE_METHODS void RQARTTCPServer_Actor::transition7_False( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	RQART_Timer.informIn(RTTimespec(0, 10000000));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction1:TestCompleted'
INLINE_METHODS void RQARTTCPServer_Actor::transition8_TestCompleted( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	SendInt(TEST_COMPLETED, 0);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction4:TestTimedOut'
INLINE_METHODS void RQARTTCPServer_Actor::transition9_TestTimedOut( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	SendInt(TEST_TIMED_OUT, 0);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction6:TestCleanedUp'
INLINE_METHODS void RQARTTCPServer_Actor::transition10_TestCleanedUp( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	SendInt(TEST_CLEANED_UP, 0);

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction3:LogMessage'
INLINE_METHODS void RQARTTCPServer_Actor::transition11_LogMessage( const RTString * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	int nSize = 0;
	char *pChar;

	pChar = ((RTString *)rtdata)->Contents;
	nSize = RTMemoryUtil::strlen(pChar);
	SendChars(TS_LOG_MESSAGE, pChar, nSize);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction12:TestFailed'
INLINE_METHODS void RQARTTCPServer_Actor::transition12_TestFailed( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	SendInt(TEST_FAILED, *rtdata);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction14:CompareFailure'
INLINE_METHODS void RQARTTCPServer_Actor::transition13_CompareFailure( const RTString * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	int nSize = 0;
	char *pChar;

	pChar = ((RTString *)rtdata)->Contents;
	nSize = RTMemoryUtil::strlen(pChar);
	SendChars(TS_COMPARE_FAILURE, pChar, nSize);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction8:timeout'
INLINE_METHODS void RQARTTCPServer_Actor::transition14_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	processIPCData();

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Operational:Junction16:ResetCompleted'
INLINE_METHODS void RQARTTCPServer_Actor::transition15_ResetCompleted( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	SendInt(RESET_COMPLETED, 0);
	// }}}USR
}
// }}}RME

INLINE_CHAINS void RQARTTCPServer_Actor::chain2_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain8_TestCompleted( void )
{
	// transition ':TOP:Operational:Junction1:TestCompleted'
	rtgChainBegin( 2, "TestCompleted" );
	rtgTransitionBegin();
	transition8_TestCompleted( msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain9_TestTimedOut( void )
{
	// transition ':TOP:Operational:Junction4:TestTimedOut'
	rtgChainBegin( 2, "TestTimedOut" );
	rtgTransitionBegin();
	transition9_TestTimedOut( msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain12_TestFailed( void )
{
	// transition ':TOP:Operational:Junction12:TestFailed'
	rtgChainBegin( 2, "TestFailed" );
	rtgTransitionBegin();
	transition12_TestFailed( (const int *)msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain10_TestCleanedUp( void )
{
	// transition ':TOP:Operational:Junction6:TestCleanedUp'
	rtgChainBegin( 2, "TestCleanedUp" );
	rtgTransitionBegin();
	transition10_TestCleanedUp( msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain11_LogMessage( void )
{
	// transition ':TOP:Operational:Junction3:LogMessage'
	rtgChainBegin( 2, "LogMessage" );
	rtgTransitionBegin();
	transition11_LogMessage( (const RTString *)msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain13_CompareFailure( void )
{
	// transition ':TOP:Operational:Junction14:CompareFailure'
	rtgChainBegin( 2, "CompareFailure" );
	rtgTransitionBegin();
	transition13_CompareFailure( (const RTString *)msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain15_ResetCompleted( void )
{
	// transition ':TOP:Operational:Junction16:ResetCompleted'
	rtgChainBegin( 2, "ResetCompleted" );
	rtgTransitionBegin();
	transition15_ResetCompleted( msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain14_timeout( void )
{
	// transition ':TOP:Operational:Junction8:timeout'
	rtgChainBegin( 2, "timeout" );
	rtgTransitionBegin();
	transition14_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain3_SetPort( void )
{
	// transition ':TOP:WaitForPort:Junction2:SetPort'
	rtgChainBegin( 4, "SetPort" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_SetPort( (const int *)msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint1_HaveListener( (const int *)msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() ) )
		chain4_True();
	else
		chain1_False();
}

// {{{RME choicePoint ':TOP:HaveListener'
INLINE_METHODS int RQARTTCPServer_Actor::choicePoint1_HaveListener( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport )
{
	// {{{USR
	if(listener.create() && listener.listen(nPort))
	{
		RQART_Log.show("Rational Quality Architect RealTime Edition listening at tcp port: ");
		
		char szPort[30];
		RTFormat::_int(nPort, szPort, 30);
	   	RQART_Log.log( szPort);
		return 1;
	}
	else return 0;

	// }}}USR
}
// }}}RME

INLINE_CHAINS void RQARTTCPServer_Actor::chain4_True( void )
{
	// transition ':TOP:HaveListener:True'
	rtgChainBegin( 6, "True" );
	rtgTransitionBegin();
	transition4_True( (const int *)msg->data, (RQARTAddInProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain1_False( void )
{
	// transition ':TOP:HaveListener:False'
	rtgChainBegin( 6, "False" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain5_timeout( void )
{
	// transition ':TOP:WaitForConnection:Junction2:timeout'
	rtgChainBegin( 5, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	if( choicePoint2_HaveConnection( msg->data, (Timing::Base *)msg->sap() ) )
		chain6_True();
	else
		chain7_False();
}

// {{{RME choicePoint ':TOP:HaveConnection'
INLINE_METHODS int RQARTTCPServer_Actor::choicePoint2_HaveConnection( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	if (listener.hasData())
	{
		c_socket.acceptFrom(listener);
		return 1;
	}
	else return 0;
	// }}}USR
}
// }}}RME

INLINE_CHAINS void RQARTTCPServer_Actor::chain6_True( void )
{
	// transition ':TOP:HaveConnection:True'
	rtgChainBegin( 7, "True" );
	rtgTransitionBegin();
	transition6_True( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTTCPServer_Actor::chain7_False( void )
{
	// transition ':TOP:HaveConnection:False'
	rtgChainBegin( 7, "False" );
	rtgTransitionBegin();
	transition7_False( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

void RQARTTCPServer_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain2_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Operational'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'AddIn'
				switch( signalIndex )
				{
				case RQARTAddInProtocol::Conjugate::rti_TestCompleted:
					chain8_TestCompleted();
					return;
				case RQARTAddInProtocol::Conjugate::rti_TestTimedOut:
					chain9_TestTimedOut();
					return;
				case RQARTAddInProtocol::Conjugate::rti_TestFailed:
					chain12_TestFailed();
					return;
				case RQARTAddInProtocol::Conjugate::rti_TestCleanedUp:
					chain10_TestCleanedUp();
					return;
				case RQARTAddInProtocol::Conjugate::rti_LogMessage:
					chain11_LogMessage();
					return;
				case RQARTAddInProtocol::Conjugate::rti_CompareFailure:
					chain13_CompareFailure();
					return;
				case RQARTAddInProtocol::Conjugate::rti_ResetCompleted:
					chain15_ResetCompleted();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'RQART_Timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain14_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:Failed'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:WaitForPort'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'AddIn'
				switch( signalIndex )
				{
				case RQARTAddInProtocol::Conjugate::rti_SetPort:
					chain3_SetPort();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME state ':TOP:WaitForConnection'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'RQART_Timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain5_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * RQARTTCPServer_Actor::getActorData( void ) const
{
	return &RQARTTCPServer_Actor::rtg_class;
}

const RTActor_class RQARTTCPServer_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 5
  , RQARTTCPServer_Actor::rtg_parent_state
  , &RQARTTCPServer
  , 0
  , (const RTComponentDescriptor *)0
  , 3
  , RQARTTCPServer_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 7
  , RQARTTCPServer_Actor::rtg_RQARTTCPServer_fields
};

const RTStateId RQARTTCPServer_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
  , 1
};

const RTPortDescriptor RQARTTCPServer_Actor::rtg_ports[] =
{
	{
		"AddIn"
	  , (const char *)0
	  , &RQARTAddInProtocol::Conjugate::rt_class
	  , RTOffsetOf( RQARTTCPServer_Actor, RQARTTCPServer_Actor::AddIn )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"RQART_Timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( RQARTTCPServer_Actor, RQARTTCPServer_Actor::RQART_Timer )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Log"
	  , (const char *)0
	  , &Log::Base::rt_class
	  , RTOffsetOf( RQARTTCPServer_Actor, RQARTTCPServer_Actor::RQART_Log )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

const RTFieldDescriptor RQARTTCPServer_Actor::rtg_RQARTTCPServer_fields[] =
{
	// {{{RME classAttribute 'anBuffer'
	{
		"anBuffer"
	  , RTOffsetOf( RQARTTCPServer_Actor, anBuffer[ 0 ] )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , &rtg_tm_RQARTTCPServer_Actor_anBuffer
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nPort'
  , {
		"nPort"
	  , RTOffsetOf( RQARTTCPServer_Actor, nPort )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nSizeRemaining'
  , {
		"nSizeRemaining"
	  , RTOffsetOf( RQARTTCPServer_Actor, nSizeRemaining )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nBodyRemaining'
  , {
		"nBodyRemaining"
	  , RTOffsetOf( RQARTTCPServer_Actor, nBodyRemaining )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'readBuffer'
  , {
		"readBuffer"
	  , RTOffsetOf( RQARTTCPServer_Actor, readBuffer[ 0 ] )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_char
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , &rtg_tm_RQARTTCPServer_Actor_readBuffer
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nNums'
  , {
		"nNums"
	  , RTOffsetOf( RQARTTCPServer_Actor, nNums )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_RTulong
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nSize'
  , {
		"nSize"
	  , RTOffsetOf( RQARTTCPServer_Actor, nSize )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_RTulong
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
