// {{{RME classifier 'Logical View::VCTRL::VPOTE::Rpm_Protocol'

#ifndef Rpm_Protocol_H
#define Rpm_Protocol_H

#ifdef PRAGMA
#pragma interface "Rpm_Protocol.h"
#endif

#include <RTSystem/VANT.h>

struct Rpm_Protocol
{
	class Base : public RTRootProtocol
	{
	public:
		inline Base( void );
		inline ~Base( void );
		enum { rti_rpm_status = rtiLast_RTRootProtocol + 1 };

	protected:
		enum { rtiLast_Rpm_Protocol = rti_rpm_status };

	public:
		inline RTInSignal rpm_status( void );
		inline RTOutSignal do_rpm_up( void );
		inline RTOutSignal do_rpm_down( void );
		inline RTOutSignal request_rpm_status( void );
		inline RTOutSignal do_straight( void );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
	class Conjugate : public RTRootProtocol
	{
	public:
		inline Conjugate( void );
		inline ~Conjugate( void );
		enum
		{
			rti_do_rpm_up = rtiLast_RTRootProtocol + 1
		  , rti_do_rpm_down
		  , rti_request_rpm_status
		  , rti_do_straight
		};

	protected:
		enum { rtiLast_Rpm_Protocol = rti_do_straight };

	public:
		inline RTInSignal do_rpm_up( void );
		inline RTInSignal do_rpm_down( void );
		inline RTInSignal request_rpm_status( void );
		inline RTInSignal do_straight( void );
		inline RTOutSignal rpm_status( const int & data );
		static const RTProtocolDescriptor rt_class;

	private:
		static const RTSignalDescriptor rt_signals[];
	};
};

inline Rpm_Protocol::Base::Base( void )
	: RTRootProtocol()
{
}

inline Rpm_Protocol::Base::~Base( void )
{
}

inline RTInSignal Rpm_Protocol::Base::rpm_status( void )
{
	return RTInSignal( this, rti_rpm_status );
}

inline RTOutSignal Rpm_Protocol::Base::do_rpm_up( void )
{
	return RTOutSignal( this, Conjugate::rti_do_rpm_up, (const void *)0, &RTType_void );
}

inline RTOutSignal Rpm_Protocol::Base::do_rpm_down( void )
{
	return RTOutSignal( this, Conjugate::rti_do_rpm_down, (const void *)0, &RTType_void );
}

inline RTOutSignal Rpm_Protocol::Base::request_rpm_status( void )
{
	return RTOutSignal( this, Conjugate::rti_request_rpm_status, (const void *)0, &RTType_void );
}

inline RTOutSignal Rpm_Protocol::Base::do_straight( void )
{
	return RTOutSignal( this, Conjugate::rti_do_straight, (const void *)0, &RTType_void );
}

inline Rpm_Protocol::Conjugate::Conjugate( void )
	: RTRootProtocol()
{
}

inline Rpm_Protocol::Conjugate::~Conjugate( void )
{
}

inline RTInSignal Rpm_Protocol::Conjugate::do_rpm_up( void )
{
	return RTInSignal( this, rti_do_rpm_up );
}

inline RTInSignal Rpm_Protocol::Conjugate::do_rpm_down( void )
{
	return RTInSignal( this, rti_do_rpm_down );
}

inline RTInSignal Rpm_Protocol::Conjugate::request_rpm_status( void )
{
	return RTInSignal( this, rti_request_rpm_status );
}

inline RTInSignal Rpm_Protocol::Conjugate::do_straight( void )
{
	return RTInSignal( this, rti_do_straight );
}

inline RTOutSignal Rpm_Protocol::Conjugate::rpm_status( const int & data )
{
	return RTOutSignal( this, Base::rti_rpm_status, &data, &RTType_int );
}

#endif /* Rpm_Protocol_H */

// }}}RME
