// {{{RME classifier 'Logical View::VCTRL::VCONT::Rudder'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Rudder.h"
#endif

#include <RTSystem/VANT.h>
#include <Rudder.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_controller"
	  , &Rudder_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Rudder_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Rudder_Actor( _rts, _ref );
}

const RTActorClass Rudder =
{
	(const RTActorClass *)0
  , "Rudder"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_Rudder_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "S1"
};

#define SUPER RTActor

Rudder_Actor::Rudder_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

Rudder_Actor::~Rudder_Actor( void )
{
}

int Rudder_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_controller
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_controller;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:S1:J4359EC8D006F:request_value'
INLINE_METHODS void Rudder_Actor::transition2_request_value( const void * rtdata, Rudder_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_controller.rudder_status(rudder_status).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:S1:J4359ECC301D6:go_right'
INLINE_METHODS void Rudder_Actor::transition3_go_right( const void * rtdata, Rudder_Protocol::Conjugate * rtport )
{
	// {{{USR
	rudder_status=2;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:S1:J4359ECEF02FE:go_left'
INLINE_METHODS void Rudder_Actor::transition4_go_left( const void * rtdata, Rudder_Protocol::Conjugate * rtport )
{
	// {{{USR
	rudder_status = 0;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:S1:J4359ED190253:go_straight'
INLINE_METHODS void Rudder_Actor::transition5_go_straight( const void * rtdata, Rudder_Protocol::Conjugate * rtport )
{
	// {{{USR
	rudder_status=1;
	// }}}USR
}
// }}}RME

INLINE_CHAINS void Rudder_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Rudder_Actor::chain2_request_value( void )
{
	// transition ':TOP:S1:J4359EC8D006F:request_value'
	rtgChainBegin( 2, "request_value" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_request_value( msg->data, (Rudder_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Rudder_Actor::chain4_go_left( void )
{
	// transition ':TOP:S1:J4359ECEF02FE:go_left'
	rtgChainBegin( 2, "go_left" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_go_left( msg->data, (Rudder_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Rudder_Actor::chain3_go_right( void )
{
	// transition ':TOP:S1:J4359ECC301D6:go_right'
	rtgChainBegin( 2, "go_right" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_go_right( msg->data, (Rudder_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Rudder_Actor::chain5_go_straight( void )
{
	// transition ':TOP:S1:J4359ED190253:go_straight'
	rtgChainBegin( 2, "go_straight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_go_straight( msg->data, (Rudder_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void Rudder_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:S1'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_controller'
				switch( signalIndex )
				{
				case Rudder_Protocol::Conjugate::rti_request_rudder_status:
					chain2_request_value();
					return;
				case Rudder_Protocol::Conjugate::rti_do_rudder_left:
					chain4_go_left();
					return;
				case Rudder_Protocol::Conjugate::rti_do_rudder_right:
					chain3_go_right();
					return;
				case Rudder_Protocol::Conjugate::rti_do_rudder_straight:
					chain5_go_straight();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Rudder_Actor::getActorData( void ) const
{
	return &Rudder_Actor::rtg_class;
}

const RTActor_class Rudder_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 2
  , Rudder_Actor::rtg_parent_state
  , &Rudder
  , 0
  , (const RTComponentDescriptor *)0
  , 1
  , Rudder_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , Rudder_Actor::rtg_Rudder_fields
};

const RTStateId Rudder_Actor::rtg_parent_state[] =
{
	0
  , 1
};

const RTPortDescriptor Rudder_Actor::rtg_ports[] =
{
	{
		"to_controller"
	  , (const char *)0
	  , &Rudder_Protocol::Conjugate::rt_class
	  , RTOffsetOf( Rudder_Actor, Rudder_Actor::to_controller )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor Rudder_Actor::rtg_Rudder_fields[] =
{
	// {{{RME classAttribute 'rudder_status'
	{
		"rudder_status"
	  , RTOffsetOf( Rudder_Actor, rudder_status )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
