// {{{RME classifier 'Logical View::VSUP::VCAL::VCAL_MEAM'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VCAL_MEAM.h"
#endif

#include <RTSystem/VANT.h>
#include <VCAL_MEAM.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"vcal_interface"
	  , &vcal_protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VCAL_MEAM_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VCAL_MEAM_Actor( _rts, _ref );
}

const RTActorClass VCAL_MEAM =
{
	(const RTActorClass *)0
  , "VCAL_MEAM"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_VCAL_MEAM_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "waiting"
  , "verifyError"
  , "sendAlarm"
  , "IsFatalError"
};

#define SUPER RTActor

VCAL_MEAM_Actor::VCAL_MEAM_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, status( false )
	, contErroCritico( 0 )
	, MAXERROR( 2 )
{
}

VCAL_MEAM_Actor::~VCAL_MEAM_Actor( void )
{
}

// {{{RME operation 'VerificarErro(char)'
void VCAL_MEAM_Actor::VerificarErro( char parCodErro )
{
	// {{{USR
	// verificar severidade do erro
	status = false;
	codErro = parCodErro;

	if (codErro=='x'){
		//erro grave	
		status = true; 

		// incrementar o total de erros
		contErroCritico ++;
	}

	if (contErroCritico>=MAXERROR)
		printf("MSG_VCAL: MAX ERROR NUMBER REACHED!!!\n");
	// }}}USR
}
// }}}RME

// {{{RME operation 'EnviarAlarme()'
void VCAL_MEAM_Actor::EnviarAlarme( void )
{
	// {{{USR
	printf("MSG_VCAL: Sending alarm...\n\n");

	// mandar msg para barramento de dados
	vcal_interface.write('k').send();
	// }}}USR
}
// }}}RME

int VCAL_MEAM_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// vcal_interface
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &vcal_interface;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Initial:Initial'
INLINE_METHODS void VCAL_MEAM_Actor::transition1_Initial( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	printf("\n\n\n*** ALARM CENTRAL ACTIVATED *** \n\n");
	printf("MSG_VCAL: Now listening... \n");
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:waiting:J43539E990197:verifyEvent'
INLINE_METHODS void VCAL_MEAM_Actor::transition2_verifyEvent( const char * rtdata, vcal_protocol::Base * rtport )
{
	// {{{USR
	printf("\n\nMSG_VCAL: ERROR DETECTED...\n");
	printf("MSG_VCAL: Verifing...\n");

	// chamar metodo
	VerificarErro(*rtdata);

	// disparar o timer
	timer.informIn(RTTimespec(1,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:IsFatalError:True'
INLINE_METHODS void VCAL_MEAM_Actor::transition4_True( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	printf("MSG_VCAL: CRITICAL ERROR!!!\n");

	// chamar o metodo
	EnviarAlarme();

	// disparar o timer
	timer.informIn(RTTimespec(1,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:IsFatalError:False'
INLINE_METHODS void VCAL_MEAM_Actor::transition5_False( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	printf("MSG_VCAL: Not critical...\n");
	printf("MSG_VCAL: Critical Errors: %d\n",contErroCritico);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:sendAlarm:J43539EBF03AB:timeout'
INLINE_METHODS void VCAL_MEAM_Actor::transition6_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	printf("MSG_VCAL: Critical Errors: %d\n",contErroCritico);
	// }}}USR
}
// }}}RME

INLINE_CHAINS void VCAL_MEAM_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	transition1_Initial( msg->data, msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCAL_MEAM_Actor::chain2_verifyEvent( void )
{
	// transition ':TOP:waiting:J43539E990197:verifyEvent'
	rtgChainBegin( 2, "verifyEvent" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_verifyEvent( (const char *)msg->data, (vcal_protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void VCAL_MEAM_Actor::chain3_timeout( void )
{
	// transition ':TOP:verifyError:J43539EAE03BF:timeout'
	rtgChainBegin( 3, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	if( choicePoint1_IsFatalError( msg->data, (Timing::Base *)msg->sap() ) )
		chain4_True();
	else
		chain5_False();
}

// {{{RME choicePoint ':TOP:IsFatalError'
INLINE_METHODS int VCAL_MEAM_Actor::choicePoint1_IsFatalError( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	return(status);
	// }}}USR
}
// }}}RME

INLINE_CHAINS void VCAL_MEAM_Actor::chain4_True( void )
{
	// transition ':TOP:IsFatalError:True'
	rtgChainBegin( 5, "True" );
	rtgTransitionBegin();
	transition4_True( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void VCAL_MEAM_Actor::chain5_False( void )
{
	// transition ':TOP:IsFatalError:False'
	rtgChainBegin( 5, "False" );
	rtgTransitionBegin();
	transition5_False( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCAL_MEAM_Actor::chain6_timeout( void )
{
	// transition ':TOP:sendAlarm:J43539EBF03AB:timeout'
	rtgChainBegin( 4, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void VCAL_MEAM_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:waiting'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'vcal_interface'
				switch( signalIndex )
				{
				case vcal_protocol::Base::rti_informError:
					chain2_verifyEvent();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:verifyError'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain3_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:sendAlarm'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain6_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VCAL_MEAM_Actor::getActorData( void ) const
{
	return &VCAL_MEAM_Actor::rtg_class;
}

const RTActor_class VCAL_MEAM_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 4
  , VCAL_MEAM_Actor::rtg_parent_state
  , &VCAL_MEAM
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , VCAL_MEAM_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 4
  , VCAL_MEAM_Actor::rtg_VCAL_MEAM_fields
};

const RTStateId VCAL_MEAM_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
};

const RTPortDescriptor VCAL_MEAM_Actor::rtg_ports[] =
{
	{
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( VCAL_MEAM_Actor, VCAL_MEAM_Actor::timer )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"vcal_interface"
	  , (const char *)0
	  , &vcal_protocol::Base::rt_class
	  , RTOffsetOf( VCAL_MEAM_Actor, VCAL_MEAM_Actor::vcal_interface )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor VCAL_MEAM_Actor::rtg_VCAL_MEAM_fields[] =
{
	// {{{RME classAttribute 'status'
	{
		"status"
	  , RTOffsetOf( VCAL_MEAM_Actor, status )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_bool
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'contErroCritico'
  , {
		"contErroCritico"
	  , RTOffsetOf( VCAL_MEAM_Actor, contErroCritico )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'codErro'
  , {
		"codErro"
	  , RTOffsetOf( VCAL_MEAM_Actor, codErro )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_char
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'MAXERROR'
  , {
		"MAXERROR"
	  , RTOffsetOf( VCAL_MEAM_Actor, MAXERROR )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
