// {{{RME classifier 'Logical View::VCNS::VCNS_Main'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VCNS_Main.h"
#endif

#include <RTSystem/VANT.h>
#include <VCNS_Main.h>
#include <COM_PROTOCOL.h>
#include <NAV_PROTOCOL.h>
#include <VIG_PROTOCOL.h>
extern const RTActorClass Cap_VNAV;
extern const RTActorClass cap_VCOM;
extern const RTActorClass cap_VVIG;

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_intbus"
	  , &VIG_PROTOCOL::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"com_to_bus"
	  , &COM_PROTOCOL::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_sat"
	  , &COM_PROTOCOL::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_station"
	  , &COM_PROTOCOL::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_bus"
	  , &NAV_PROTOCOL::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VCNS_Main_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VCNS_Main_Actor( _rts, _ref );
}

const RTActorClass VCNS_Main =
{
	(const RTActorClass *)0
  , "VCNS_Main"
  , (RTVersionId)0
  , 5
  , rtg_relays
  , new_VCNS_Main_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
};

static const RTInterfaceDescriptor rtg_interfaces_cap_vvig[] =
{
	{
		"to_intbus"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_cap_vvig[] =
{
	{
		0
	  , &VIG_PROTOCOL::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_cap_vcom[] =
{
	{
		"to_sat"
	  , 1
	}
  , {
		"com_to_bus"
	  , 1
	}
  , {
		"to_station"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_cap_vcom[] =
{
	{
		0
	  , &COM_PROTOCOL::Conjugate::rt_class
	}
  , {
		1
	  , &COM_PROTOCOL::Conjugate::rt_class
	}
  , {
		2
	  , &COM_PROTOCOL::Conjugate::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_cap_vnav[] =
{
	{
		"to_bus"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_cap_vnav[] =
{
	{
		0
	  , &NAV_PROTOCOL::Base::rt_class
	}
};

#define SUPER RTActor

VCNS_Main_Actor::VCNS_Main_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

VCNS_Main_Actor::~VCNS_Main_Actor( void )
{
}

int VCNS_Main_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_intbus
		if( rtg_repIndex < 1 )
		{
			// cap_vvig/to_intbus
			return cap_vvig._followIn( rtg_end, 0, rtg_repIndex );
		}
		break;
	case 1:
		// com_to_bus
		if( rtg_repIndex < 1 )
		{
			// cap_vcom/com_to_bus
			return cap_vcom._followIn( rtg_end, 1, rtg_repIndex );
		}
		break;
	case 2:
		// to_sat
		if( rtg_repIndex < 1 )
		{
			// cap_vcom/to_sat
			return cap_vcom._followIn( rtg_end, 0, rtg_repIndex );
		}
		break;
	case 3:
		// to_station
		if( rtg_repIndex < 1 )
		{
			// cap_vcom/to_station
			return cap_vcom._followIn( rtg_end, 2, rtg_repIndex );
		}
		break;
	case 4:
		// to_bus
		if( rtg_repIndex < 1 )
		{
			// cap_vnav/to_bus
			return cap_vnav._followIn( rtg_end, 0, rtg_repIndex );
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

int VCNS_Main_Actor::_followOutV( RTBindingEnd & rtg_end, int rtg_compId, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_compId )
	{
	case 1:
		// cap_vvig
		switch( rtg_portId )
		{
		case 0:
			// to_intbus
			if( rtg_repIndex < 1 )
			{
				// to_intbus
				return _followOut( rtg_end, 0, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 2:
		// cap_vcom
		switch( rtg_portId )
		{
		case 0:
			// to_sat
			if( rtg_repIndex < 1 )
			{
				// to_sat
				return _followOut( rtg_end, 2, rtg_repIndex );
			}
			break;
		case 1:
			// com_to_bus
			if( rtg_repIndex < 1 )
			{
				// com_to_bus
				return _followOut( rtg_end, 1, rtg_repIndex );
			}
			break;
		case 2:
			// to_station
			if( rtg_repIndex < 1 )
			{
				// to_station
				return _followOut( rtg_end, 3, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 3:
		// cap_vnav
		switch( rtg_portId )
		{
		case 0:
			// to_bus
			if( rtg_repIndex < 1 )
			{
				// to_bus
				return _followOut( rtg_end, 4, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	default:
		break;
	}
	return RTActor::_followOutV( rtg_end, rtg_compId, rtg_portId, rtg_repIndex );
}

void VCNS_Main_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VCNS_Main_Actor::getActorData( void ) const
{
	return &VCNS_Main_Actor::rtg_class;
}

const RTActor_class VCNS_Main_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 1
  , VCNS_Main_Actor::rtg_parent_state
  , &VCNS_Main
  , 3
  , VCNS_Main_Actor::rtg_capsule_roles
  , 0
  , (const RTPortDescriptor *)0
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId VCNS_Main_Actor::rtg_parent_state[] =
{
	0
};

const RTComponentDescriptor VCNS_Main_Actor::rtg_capsule_roles[] =
{
	{
		"cap_vvig"
	  , &cap_VVIG
	  , RTOffsetOf( VCNS_Main_Actor, cap_vvig )
	  , 1
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_cap_vvig
	  , 1
	  , rtg_bindings_cap_vvig
	}
  , {
		"cap_vcom"
	  , &cap_VCOM
	  , RTOffsetOf( VCNS_Main_Actor, cap_vcom )
	  , 2
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 3
	  , rtg_interfaces_cap_vcom
	  , 3
	  , rtg_bindings_cap_vcom
	}
  , {
		"cap_vnav"
	  , &Cap_VNAV
	  , RTOffsetOf( VCNS_Main_Actor, cap_vnav )
	  , 3
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_cap_vnav
	  , 1
	  , rtg_bindings_cap_vnav
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
