// {{{RME classifier 'Logical View::VCTRL::VCOMB::VCOMB'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VCOMB.h"
#endif

#include <RTSystem/VANT.h>
#include <VCOMB.h>
#include <Bus_Comb_Protocol.h>
#include <Consume_Protocol.h>
#include <Level_Protocol.h>
#include <Mix_Protocol.h>
#include <Power_Comb_Protocol.h>
extern const RTActorClass Consume;
extern const RTActorClass FuelController;
extern const RTActorClass Level;
extern const RTActorClass Mix;

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"bus_port"
	  , &Bus_Comb_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"power_port"
	  , &Power_Comb_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VCOMB_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VCOMB_Actor( _rts, _ref );
}

const RTActorClass VCOMB =
{
	(const RTActorClass *)0
  , "VCOMB"
  , (RTVersionId)0
  , 2
  , rtg_relays
  , new_VCOMB_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
};

static const RTInterfaceDescriptor rtg_interfaces_fuelController[] =
{
	{
		"to_bus"
	  , 1
	}
  , {
		"to_power"
	  , 1
	}
  , {
		"to_consume"
	  , 1
	}
  , {
		"to_mix"
	  , 1
	}
  , {
		"to_level"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_fuelController[] =
{
	{
		0
	  , &Bus_Comb_Protocol::Base::rt_class
	}
  , {
		1
	  , &Power_Comb_Protocol::Base::rt_class
	}
  , {
		2
	  , &Consume_Protocol::Conjugate::rt_class
	}
  , {
		3
	  , &Mix_Protocol::Conjugate::rt_class
	}
  , {
		4
	  , &Level_Protocol::Conjugate::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_consume[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_consume[] =
{
	{
		0
	  , &Consume_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_mix[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_mix[] =
{
	{
		0
	  , &Mix_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_level[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_level[] =
{
	{
		0
	  , &Level_Protocol::Base::rt_class
	}
};

#define SUPER RTActor

VCOMB_Actor::VCOMB_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

VCOMB_Actor::~VCOMB_Actor( void )
{
}

int VCOMB_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// bus_port
		if( rtg_repIndex < 1 )
		{
			// fuelController/to_bus
			return fuelController._followIn( rtg_end, 0, rtg_repIndex );
		}
		break;
	case 1:
		// power_port
		if( rtg_repIndex < 1 )
		{
			// fuelController/to_power
			return fuelController._followIn( rtg_end, 1, rtg_repIndex );
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

int VCOMB_Actor::_followOutV( RTBindingEnd & rtg_end, int rtg_compId, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_compId )
	{
	case 1:
		// fuelController
		switch( rtg_portId )
		{
		case 0:
			// to_bus
			if( rtg_repIndex < 1 )
			{
				// bus_port
				return _followOut( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 1:
			// to_power
			if( rtg_repIndex < 1 )
			{
				// power_port
				return _followOut( rtg_end, 1, rtg_repIndex );
			}
			break;
		case 2:
			// to_consume
			if( rtg_repIndex < 1 )
			{
				// consume/to_controller
				return consume._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 3:
			// to_mix
			if( rtg_repIndex < 1 )
			{
				// mix/to_controller
				return mix._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 4:
			// to_level
			if( rtg_repIndex < 1 )
			{
				// level/to_controller
				return level._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 2:
		// consume
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// fuelController/to_consume
				return fuelController._followIn( rtg_end, 2, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 3:
		// mix
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// fuelController/to_mix
				return fuelController._followIn( rtg_end, 3, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 4:
		// level
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// fuelController/to_level
				return fuelController._followIn( rtg_end, 4, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	default:
		break;
	}
	return RTActor::_followOutV( rtg_end, rtg_compId, rtg_portId, rtg_repIndex );
}

void VCOMB_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VCOMB_Actor::getActorData( void ) const
{
	return &VCOMB_Actor::rtg_class;
}

const RTActor_class VCOMB_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 1
  , VCOMB_Actor::rtg_parent_state
  , &VCOMB
  , 4
  , VCOMB_Actor::rtg_capsule_roles
  , 0
  , (const RTPortDescriptor *)0
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId VCOMB_Actor::rtg_parent_state[] =
{
	0
};

const RTComponentDescriptor VCOMB_Actor::rtg_capsule_roles[] =
{
	{
		"fuelController"
	  , &FuelController
	  , RTOffsetOf( VCOMB_Actor, fuelController )
	  , 1
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 5
	  , rtg_interfaces_fuelController
	  , 5
	  , rtg_bindings_fuelController
	}
  , {
		"consume"
	  , &Consume
	  , RTOffsetOf( VCOMB_Actor, consume )
	  , 2
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_consume
	  , 1
	  , rtg_bindings_consume
	}
  , {
		"mix"
	  , &Mix
	  , RTOffsetOf( VCOMB_Actor, mix )
	  , 3
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_mix
	  , 1
	  , rtg_bindings_mix
	}
  , {
		"level"
	  , &Level
	  , RTOffsetOf( VCOMB_Actor, level )
	  , 4
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_level
	  , 1
	  , rtg_bindings_level
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
