// {{{RME classifier 'Logical View::VCNS::VCOM::VCOM_Controller'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VCOM_Controller.h"
#endif

#include <RTSystem/VANT.h>
#include <VCOM_Controller.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_sat"
	  , &COM_PROTOCOL::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_bus"
	  , &COM_PROTOCOL::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_station"
	  , &COM_PROTOCOL::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VCOM_Controller_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VCOM_Controller_Actor( _rts, _ref );
}

const RTActorClass VCOM_Controller =
{
	(const RTActorClass *)0
  , "VCOM_Controller"
  , (RTVersionId)0
  , 3
  , rtg_relays
  , new_VCOM_Controller_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Waiting"
};

#define SUPER RTActor

VCOM_Controller_Actor::VCOM_Controller_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, intMission( 1 )
{
}

VCOM_Controller_Actor::~VCOM_Controller_Actor( void )
{
}

int VCOM_Controller_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_sat
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_sat;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// to_bus
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_bus;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 2:
		// to_station
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_station;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Waiting:J436F577C039B:fuel_level'
INLINE_METHODS void VCOM_Controller_Actor::transition2_fuel_level( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_sat.fuel_level(*rtdata).send();
	//to_station.fuel_level(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F59470204:speed_value'
INLINE_METHODS void VCOM_Controller_Actor::transition3_speed_value( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_sat.speed_value(*rtdata).send();
	//to_station.speed_value(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F5FEE02DE:position_value'
INLINE_METHODS void VCOM_Controller_Actor::transition4_position_value( const Coordinate * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_sat.positioning_value(*rtdata).send();
	//to_station.positioning_value(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F8361015B:traj_data'
INLINE_METHODS void VCOM_Controller_Actor::transition5_traj_data( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_sat.traj_data(*rtdata).send();
	//to_station.traj_data(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F83F500FC:image_data'
INLINE_METHODS void VCOM_Controller_Actor::transition6_image_data( const rgb * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_sat.image_data(*rtdata).send();
	//to_station.image_data(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F84330267:alarm_data'
INLINE_METHODS void VCOM_Controller_Actor::transition7_alarm_data( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_sat.alarm_data(*rtdata).send();
	//to_station.alarm_data(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F848300E6:flight_data'
INLINE_METHODS void VCOM_Controller_Actor::transition8_flight_data( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_sat.flight_data(*rtdata).send();
	//to_station.flight_data(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F869701AC:req_fuel_lv'
INLINE_METHODS void VCOM_Controller_Actor::transition9_req_fuel_lv( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_bus.request_fuel_level(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F87180049:req_traj_dt'
INLINE_METHODS void VCOM_Controller_Actor::transition10_req_traj_dt( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_bus.request_traj_data(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F889501D9:req_posit_vl'
INLINE_METHODS void VCOM_Controller_Actor::transition11_req_posit_vl( const Coordinate * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_bus.request_positioning_value(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F89F10002:req_speed_vl'
INLINE_METHODS void VCOM_Controller_Actor::transition12_req_speed_vl( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_bus.request_speed_value(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F8ABF0290:req_image_dt'
INLINE_METHODS void VCOM_Controller_Actor::transition13_req_image_dt( const rgb * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_bus.request_image_data(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F8AFA0377:req_flight_dt'
INLINE_METHODS void VCOM_Controller_Actor::transition14_req_flight_dt( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	//to_bus.request_flight_data(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F8B3D03C6:final_posit'
INLINE_METHODS void VCOM_Controller_Actor::transition15_final_posit( const Coordinate * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	to_bus.final_position(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F8B9D0089:mission_speed'
INLINE_METHODS void VCOM_Controller_Actor::transition16_mission_speed( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	to_bus.mission_speed(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J436F8CA402D8:mission_time'
INLINE_METHODS void VCOM_Controller_Actor::transition17_mission_time( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	to_bus.mission_time(*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Waiting:J437F88B400DC:step1Mission'
INLINE_METHODS void VCOM_Controller_Actor::transition18_step1Mission( const int * rtdata, COM_PROTOCOL::Base * rtport )
{
	// {{{USR
	// enviar trajetoria planejada para VSUP
	printf("\n*** MSG_VCNS_Communication: MISSION #%d HAS BEEN ACCEPTED ***\n", intMission);
	intMission++;

	to_bus.toFlightControl('1').send();
	printf("MSG_VCNS_Communication: Message to VCNS sent by VSUP\n\n");
	// }}}USR
}
// }}}RME

INLINE_CHAINS void VCOM_Controller_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain9_req_fuel_lv( void )
{
	// transition ':TOP:Waiting:J436F869701AC:req_fuel_lv'
	rtgChainBegin( 2, "req_fuel_lv" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition9_req_fuel_lv( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain10_req_traj_dt( void )
{
	// transition ':TOP:Waiting:J436F87180049:req_traj_dt'
	rtgChainBegin( 2, "req_traj_dt" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition10_req_traj_dt( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain11_req_posit_vl( void )
{
	// transition ':TOP:Waiting:J436F889501D9:req_posit_vl'
	rtgChainBegin( 2, "req_posit_vl" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition11_req_posit_vl( (const Coordinate *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain12_req_speed_vl( void )
{
	// transition ':TOP:Waiting:J436F89F10002:req_speed_vl'
	rtgChainBegin( 2, "req_speed_vl" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition12_req_speed_vl( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain13_req_image_dt( void )
{
	// transition ':TOP:Waiting:J436F8ABF0290:req_image_dt'
	rtgChainBegin( 2, "req_image_dt" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition13_req_image_dt( (const rgb *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain14_req_flight_dt( void )
{
	// transition ':TOP:Waiting:J436F8AFA0377:req_flight_dt'
	rtgChainBegin( 2, "req_flight_dt" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition14_req_flight_dt( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain16_mission_speed( void )
{
	// transition ':TOP:Waiting:J436F8B9D0089:mission_speed'
	rtgChainBegin( 2, "mission_speed" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition16_mission_speed( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain17_mission_time( void )
{
	// transition ':TOP:Waiting:J436F8CA402D8:mission_time'
	rtgChainBegin( 2, "mission_time" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition17_mission_time( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain15_final_posit( void )
{
	// transition ':TOP:Waiting:J436F8B3D03C6:final_posit'
	rtgChainBegin( 2, "final_posit" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition15_final_posit( (const Coordinate *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain2_fuel_level( void )
{
	// transition ':TOP:Waiting:J436F577C039B:fuel_level'
	rtgChainBegin( 2, "fuel_level" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_fuel_level( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain5_traj_data( void )
{
	// transition ':TOP:Waiting:J436F8361015B:traj_data'
	rtgChainBegin( 2, "traj_data" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_traj_data( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain4_position_value( void )
{
	// transition ':TOP:Waiting:J436F5FEE02DE:position_value'
	rtgChainBegin( 2, "position_value" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_position_value( (const Coordinate *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain3_speed_value( void )
{
	// transition ':TOP:Waiting:J436F59470204:speed_value'
	rtgChainBegin( 2, "speed_value" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_speed_value( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain6_image_data( void )
{
	// transition ':TOP:Waiting:J436F83F500FC:image_data'
	rtgChainBegin( 2, "image_data" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_image_data( (const rgb *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain7_alarm_data( void )
{
	// transition ':TOP:Waiting:J436F84330267:alarm_data'
	rtgChainBegin( 2, "alarm_data" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition7_alarm_data( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain8_flight_data( void )
{
	// transition ':TOP:Waiting:J436F848300E6:flight_data'
	rtgChainBegin( 2, "flight_data" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition8_flight_data( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VCOM_Controller_Actor::chain18_step1Mission( void )
{
	// transition ':TOP:Waiting:J437F88B400DC:step1Mission'
	rtgChainBegin( 2, "step1Mission" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition18_step1Mission( (const int *)msg->data, (COM_PROTOCOL::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void VCOM_Controller_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Waiting'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_sat'
				switch( signalIndex )
				{
				case COM_PROTOCOL::Base::rti_request_fuel_level:
					chain9_req_fuel_lv();
					return;
				case COM_PROTOCOL::Base::rti_request_traj_data:
					chain10_req_traj_dt();
					return;
				case COM_PROTOCOL::Base::rti_request_positioning_value:
					chain11_req_posit_vl();
					return;
				case COM_PROTOCOL::Base::rti_request_speed_value:
					chain12_req_speed_vl();
					return;
				case COM_PROTOCOL::Base::rti_request_image_data:
					chain13_req_image_dt();
					return;
				case COM_PROTOCOL::Base::rti_request_flight_data:
					chain14_req_flight_dt();
					return;
				case COM_PROTOCOL::Base::rti_mission_speed:
					chain16_mission_speed();
					return;
				case COM_PROTOCOL::Base::rti_mission_time:
					chain17_mission_time();
					return;
				case COM_PROTOCOL::Base::rti_final_position:
					chain15_final_posit();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'to_bus'
				switch( signalIndex )
				{
				case COM_PROTOCOL::Base::rti_request_fuel_level:
					chain2_fuel_level();
					return;
				case COM_PROTOCOL::Base::rti_request_traj_data:
					chain5_traj_data();
					return;
				case COM_PROTOCOL::Base::rti_request_positioning_value:
					chain4_position_value();
					return;
				case COM_PROTOCOL::Base::rti_request_speed_value:
					chain3_speed_value();
					return;
				case COM_PROTOCOL::Base::rti_request_image_data:
					chain6_image_data();
					return;
				case COM_PROTOCOL::Base::rti_request_alarm_data:
					chain7_alarm_data();
					return;
				case COM_PROTOCOL::Base::rti_request_flight_data:
					chain8_flight_data();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 3:
				// {{{RME port 'to_station'
				switch( signalIndex )
				{
				case COM_PROTOCOL::Base::rti_request_fuel_level:
					chain9_req_fuel_lv();
					return;
				case COM_PROTOCOL::Base::rti_request_traj_data:
					chain18_step1Mission();
					return;
				case COM_PROTOCOL::Base::rti_request_positioning_value:
					chain11_req_posit_vl();
					return;
				case COM_PROTOCOL::Base::rti_request_speed_value:
					chain12_req_speed_vl();
					return;
				case COM_PROTOCOL::Base::rti_request_image_data:
					chain13_req_image_dt();
					return;
				case COM_PROTOCOL::Base::rti_request_flight_data:
					chain14_req_flight_dt();
					return;
				case COM_PROTOCOL::Base::rti_mission_speed:
					chain16_mission_speed();
					return;
				case COM_PROTOCOL::Base::rti_mission_time:
					chain17_mission_time();
					return;
				case COM_PROTOCOL::Base::rti_final_position:
					chain15_final_posit();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VCOM_Controller_Actor::getActorData( void ) const
{
	return &VCOM_Controller_Actor::rtg_class;
}

const RTActor_class VCOM_Controller_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 2
  , VCOM_Controller_Actor::rtg_parent_state
  , &VCOM_Controller
  , 0
  , (const RTComponentDescriptor *)0
  , 3
  , VCOM_Controller_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , VCOM_Controller_Actor::rtg_VCOM_Controller_fields
};

const RTStateId VCOM_Controller_Actor::rtg_parent_state[] =
{
	0
  , 1
};

const RTPortDescriptor VCOM_Controller_Actor::rtg_ports[] =
{
	{
		"to_sat"
	  , (const char *)0
	  , &COM_PROTOCOL::Base::rt_class
	  , RTOffsetOf( VCOM_Controller_Actor, VCOM_Controller_Actor::to_sat )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_bus"
	  , (const char *)0
	  , &COM_PROTOCOL::Base::rt_class
	  , RTOffsetOf( VCOM_Controller_Actor, VCOM_Controller_Actor::to_bus )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_station"
	  , (const char *)0
	  , &COM_PROTOCOL::Base::rt_class
	  , RTOffsetOf( VCOM_Controller_Actor, VCOM_Controller_Actor::to_station )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor VCOM_Controller_Actor::rtg_VCOM_Controller_fields[] =
{
	// {{{RME classAttribute 'intMission'
	{
		"intMission"
	  , RTOffsetOf( VCOM_Controller_Actor, intMission )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
